@@ -137,21 +137,25 @@ find_info_part (gchar *part_name, gchar *base)
    */
   gchar *path;
   gchar *tmp;
-  gchar *bzfname, *gzfname;
+  gchar *bzfname, *gzfname, *lzfd;
   gchar *uri = NULL;
   tmp = g_strrstr (base, "/");
   path = g_strndup (base, tmp-base);
 
   bzfname = g_strconcat (path, "/", part_name, ".bz2", NULL);
   gzfname = g_strconcat (path, "/", part_name, ".gz", NULL);
+  lzfd = g_strconcat (path, "/", part_name, ".lzma", NULL);
   
   if (g_file_test (bzfname, G_FILE_TEST_EXISTS))
     uri = g_strdup (bzfname);
   else if (g_file_test (gzfname, G_FILE_TEST_EXISTS))
     uri = g_strdup (gzfname);
+  else if (g_file_test (lzfd, G_FILE_TEST_EXISTS))
+    uri = g_strdup (lzfd);
 
   g_free (bzfname);
   g_free (gzfname);
+  g_free (lzfd);
   g_free (path);
   return uri;
 
@@ -184,10 +188,16 @@ static char
 		  filename = find_info_part (items[0], file);
 		  str = open_info_file (filename);
 		  if (!str) {
-		  	return NULL;
+			g_strfreev (items);
+		  	continue;
 		  }
 			pages = g_strsplit (str, "", 2);
 			g_free (str);
+			if (!pages[1]) {
+				g_strfreev (items);
+				g_strfreev (pages);
+		  		continue;
+			}
 
 			offset =  atoi(items[1]);
 			plength = strlen(pages[1]);
@@ -354,6 +364,12 @@ GtkTreeIter * find_real_sibling (GtkTreeModel *model,
       title = gtk_tree_model_get_string_from_iter (model, r);
   }
 
+  if (!g_str_equal (title, reftitle))
+    {
+      gtk_tree_iter_free (tmp);
+      tmp = NULL;
+    }
+
   gtk_tree_iter_free (r);
   g_free (title);
   g_free (reftitle);
@@ -403,7 +419,7 @@ process_page (GtkTreeStore *tree, GHashTable *nodes2offsets,
 	
 	debug_print (DB_DEBUG, "-- Processing Page %s\n\tParent: %s\n", node, up);
 
-	iter = g_malloc0 (sizeof (GtkTreeIter));
+	iter = g_slice_alloc0 (sizeof (GtkTreeIter));
 	/* check to see if we need to process our parent and siblings */
 	if (up && g_ascii_strncasecmp (up, "(dir)", 5) && strcmp (up, "Top"))
 	{
