@@ -27,7 +27,6 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlreader.h>
@@ -41,6 +40,7 @@
 #include <libxslt/extensions.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/xsltutils.h>
+#define I_KNOW_RARIAN_0_8_IS_UNSTABLE
 #include <rarian.h>
 
 #ifdef ENABLE_BEAGLE
@@ -173,6 +173,7 @@ static void          search_free_container     (SearchContainer       *c);
 #ifdef ENABLE_BEAGLE
 static BeagleClient   *beagle_client;
 #endif /* ENABLE_BEAGLE */
+static char const * const * langs;
 
 YelpSearchParser *
 yelp_search_parser_new (void)
@@ -233,8 +234,19 @@ yelp_search_parser_process (YelpSearchParser *parser, gchar *search_terms)
 {
     debug_print (DB_FUNCTION, "entering\n");
 
+    langs = g_get_language_names ();
+
     parser->search_terms = decode_uri (search_terms);
 
+#ifdef ENABLE_BEAGLE
+    if (beagle_util_daemon_is_running()) {
+	beagle_client = beagle_client_new (NULL);
+	debug_print (DB_DEBUG, "client: %p\n", beagle_client);
+    } else {
+	beagle_client = NULL;
+    }
+#endif /* ENABLE_BEAGLE */
+
     parser->search_process_id = 
 	g_idle_add_full (G_PRIORITY_LOW,
 			 (GSourceFunc) search_parser_process_idle,
@@ -301,7 +313,8 @@ check_finished (YelpSearchParser *parser)
     g_free (check);
     xmlNewTextChild (parser->root, NULL, BAD_CAST "online1", BAD_CAST split[1]);
 
-    parser->finished = TRUE;
+    if (parser->snippet_request_count == 0)
+      parser->finished = TRUE;
 
 }
 
@@ -354,7 +367,7 @@ snippet_response (BeagleSnippetRequest *request, BeagleSnippetResponse *response
 
     const char *xml = beagle_snippet_response_get_snippet (response);
 
-    if (xml == NULL) {
+    if (xml == NULL || strlen (xml) == 0) {
 	debug_print (DB_DEBUG, "snippet_response empty\n");
 	return;
     }
@@ -396,7 +409,8 @@ hits_added_cb (BeagleQuery *query, BeagleHitsAddedResponse *response, YelpSearch
 }
 
 static gboolean
-check_lang (const char *lang) {
+check_lang (const char *lang) 
+{
     int i;
     for (i = 0; langs[i]; i++) {
 	if (!strncmp (lang, langs[i], 2)) {
@@ -526,13 +540,29 @@ search_parser_process_idle (YelpSearchParser *parser)
     xmlSetProp (parser->root, BAD_CAST "title", BAD_CAST parser->search_terms);
     xmlDocSetRootElement (parser->search_doc, parser->root);
 
+
+    /* Easter egg.  Do not translate.  Oh, and congratulations on
+     * finding it ;)
+     */
+    if (g_str_equal (parser->search_terms, "Easter Egg")) {
+	xmlNode *child;
+	child = xmlNewTextChild (parser->root, NULL,
+				 BAD_CAST "result", NULL);
+	xmlSetProp (child, BAD_CAST "uri", BAD_CAST "http://www.gnome.org");
+	xmlSetProp (child, BAD_CAST "title", BAD_CAST "Congratulations!");
+	xmlNewChild (child, NULL, BAD_CAST "snippet",
+		     BAD_CAST "It's widely known GNOME rocks.  Welcome to the party.");
+	xmlNewChild (child, NULL, BAD_CAST "score",
+		     BAD_CAST "1000.0");
+    }
+
 #ifdef ENABLE_BEAGLE
     if (beagle_client != NULL) {
 	query = beagle_query_new ();
 
 	beagle_query_set_max_hits (query, 10000);
 	beagle_query_add_text (query, parser->search_terms);
-	beagle_query_add_source (query, "documentation");
+	beagle_query_add_text (query, "source:documentation OR source:manpages");
 
 	parser->hits = g_ptr_array_new ();
 
@@ -561,9 +591,13 @@ search_parser_process_idle (YelpSearchParser *parser)
 #endif
 	g_return_val_if_fail (parser->slow_search_setup_process_id == 0, FALSE);
 
-	parser->slow_search_setup_process_id = 
-	    g_idle_add ((GSourceFunc) slow_search_setup,
-		        parser);
+	if (g_str_equal (parser->search_terms, ":"))
+	    check_finished(parser);
+	else {
+	    parser->slow_search_setup_process_id = 
+		g_idle_add ((GSourceFunc) slow_search_setup,
+			    parser);
+	}
 #ifdef ENABLE_BEAGLE
     }
 #endif
@@ -689,13 +723,15 @@ void s_characters(void * data,
 	gint i = 0;
 	gchar *s_term = c->search_term[i];
 	while (s_term && c->score_per_word[i] < 1.0) {
+	    gchar *location;
+
 	    if (c->stop_word[i] || c->score_per_word[c->dup_of[i]] == 1.0) {
 		i++;
 		s_term = c->search_term[i];
 		continue;
 	    }
 
-	    gchar *location = strstr (tmp, s_term);	    
+	    location = strstr (tmp, s_term);	    
 	    if (location) {
 		gchar before = *(location-1);
 		gchar after = *(location+strlen(s_term));
@@ -857,7 +893,7 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
      * to words.  Things that may be put at ends of words to slightly 
      * alter their meaning (like -ing and -s in English).  This is a
      * colon seperated list (I like colons).  If there are none,
-     * please use the strig NULL.  If there is only 1, please
+     * please use the string NULL.  If there is only 1, please
      * add a colon at the end of the list
      */
     common_suffixes = g_strdup (_("ers:er:ing:es:s:'s"));
@@ -873,6 +909,7 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
     } else {
 	suffixes = NULL;
     }
+
     search_terms = g_strdelimit (search_terms, ":", ' ');
     list_copy = g_strsplit (g_utf8_casefold (g_strstrip (
 					  search_terms), -1),
@@ -949,7 +986,10 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
     (*dups) = g_new0 (gint, n_terms);
     (*stops) = g_new0 (gboolean, n_terms);
     list_copy = g_strsplit (dup_str, ":", -1);
-    
+
+    if (n_terms == 0)
+	goto done;
+	
     for (iter = *terms; *iter; iter++) {
 	i++;
 	if (g_str_equal (list_copy[i], "O")) {
@@ -966,6 +1006,7 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
 	}
     }
 
+ done:
     /* Clean up all those pesky strings */
     g_free (ignore_words);
     g_free (common_prefixes);
@@ -994,6 +1035,9 @@ slow_search_setup (YelpSearchParser *parser)
    terms_number = build_lists (parser->search_terms,&terms_list, 
 				&dup_list, &stop_list, 
 				&required_no);
+   if (terms_number < 1)
+       goto done;
+       
    data = g_new0 (SearchDocData, 1);
    data->container = g_new0 (SearchContainer, 1);
    data->parser = parser;
@@ -1008,7 +1052,8 @@ slow_search_setup (YelpSearchParser *parser)
 
    search_process_man (parser, terms_list);
    search_process_info (parser, terms_list);
- 
+
+ done: 
    check_finished (parser);
 
    return FALSE;
@@ -1023,6 +1068,9 @@ slow_search_process (RrnReg *reg, SearchDocData *data)
     gchar *ptr, *path;
     gchar *fname;
 
+    if (!reg->uri || g_str_equal (reg->uri, ""))
+	return TRUE;
+
     /* Set up the container with the new data */
     if (g_str_has_prefix (reg->uri, "file:")) {
 	fname = &(reg->uri[5]);
@@ -1041,18 +1089,25 @@ slow_search_process (RrnReg *reg, SearchDocData *data)
     container->doc_title = g_strdup ((gchar *) reg->name);
     container->score=0;
     container->html = FALSE;
-    container->default_snippet = g_strdup ((gchar *) reg->comment);
+    if (reg->comment)
+    	container->default_snippet = g_strdup ((gchar *) reg->comment);
+    else
+    	container->default_snippet = g_strdup (_("No Comment"));
     container->current_subsection = NULL;
     container->elem_type = NULL;
 
     ptr = g_strrstr (container->base_filename, "/");
     
-    path = g_strndup (container->base_filename, 
-		      ptr - container->base_filename);
+    if (ptr) {
+	path = g_strndup (container->base_filename, 
+			  ptr - container->base_filename);
+    } else {
+	path = g_strdup (container->base_filename);
+    }
     
     /* BEGIN HTML special block */
-    if (g_str_equal (reg->type, "text/html") ||
-	g_str_has_suffix (fname, "html")) {
+    if (reg->type && (g_str_equal (reg->type, "text/html") ||
+		      g_str_has_suffix (fname, "html"))) {
 	GDir *dir;
 	gchar *filename;
 	container->html = TRUE;
@@ -1158,6 +1213,7 @@ search_clean_snippet (gchar *snippet, gchar **terms)
     gboolean am_cutting = FALSE;
     gchar *result = NULL;
     gboolean found_terms = FALSE;
+    int i;
 
 
     if (!snippet)
@@ -1168,18 +1224,18 @@ search_clean_snippet (gchar *snippet, gchar **terms)
     }
     result = g_strdup (snippet);
 
-    for (iteration = terms; *iteration; iteration++) {
+    for (iteration = terms, i = 0; iteration[i]; i++) {
 	gchar *before, *after, *tmp;
 	gchar *str;
 	gchar before_c, after_c;
 	gint count = 0;
 
-	while ((str = strstr (result, (*iteration)))) {
+	while ((str = strstr (result, (iteration[i])))) {
 	    gboolean breaking = FALSE;
 	    gint i;
 	    for (i=0; i< count; i++) {
 		str++;
-		str = strstr (str, (*iteration));
+		str = strstr (str, (iteration[i]));
 		if (!str) {
 		    breaking = TRUE;
 		    break;
@@ -1190,7 +1246,7 @@ search_clean_snippet (gchar *snippet, gchar **terms)
 		break;
 
 	    before_c = *(str-1);
-	    after_c = *(str+strlen(*iteration));
+	    after_c = *(str+strlen(iteration[i]));
 
 	    if (g_ascii_isalpha (before_c) || g_ascii_isalpha (after_c)) {
 		continue;
@@ -1218,7 +1274,7 @@ search_clean_snippet (gchar *snippet, gchar **terms)
 	    before = g_strconcat (tmp, "<em>", NULL);
 	    g_free (tmp);
 	    
-	    str += strlen (*iteration);
+	    str += strlen (iteration[i]);
 
 	    if (am_cutting && !found_terms && strlen (str) > len_after_term) {
 		gchar *tmp1;
@@ -1230,7 +1286,7 @@ search_clean_snippet (gchar *snippet, gchar **terms)
 		tmp = g_strdup (str);
 	    }
 	    
-	    after = g_strconcat ((*iteration), "</em>", tmp, NULL);
+	    after = g_strconcat ((iteration[i]), "</em>", tmp, NULL);
 
 
 	    
@@ -1294,7 +1350,16 @@ process_man_result (YelpSearchParser *parser, gchar *result, gchar **terms)
 	/*gint i;*/
 
 	if (line == NULL || line[0] == NULL || line[1] == NULL)
-	    continue;
+	    goto done;
+
+	/* RH man inserts rpm references into the whatis database */
+	if (g_str_has_prefix (line[1], "rpm)"))
+            goto done;
+
+        tmp = strchr (line[0], ' ');
+        if (tmp)
+            *tmp = '\0';
+
 
 	title = g_strdup (g_strstrip (line[0]));
 	after = strstr (line[1], ")");
@@ -1322,6 +1387,7 @@ process_man_result (YelpSearchParser *parser, gchar *result, gchar **terms)
 	xmlNewChild (child, NULL, BAD_CAST "score",
 		     BAD_CAST "0.1");
 	g_free (tmp);
+    done:
 	g_strfreev (line);
     }
 
